<?php
declare(strict_types=1);

namespace Drupal\membership\Entity\Member;

use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\Link;
use Drupal\Core\Url;

/**
 * Defines the Membership Member entity.
 *
 * @ingroup membership
 *
 * @ContentEntityType(
 *   id = "membership_member",
 *   label = @Translation("Member"),
 *   label_collection = @Translation("Members"),
 *   label_singular = @Translation("member"),
 *   label_plural = @Translation("members"),
 *   label_count = @PluralTranslation(
 *     singular = "@count member",
 *     plural = "@count members"
 *   ),
 *   handlers = {
 *     "view_builder" = "Drupal\Core\Entity\EntityViewBuilder",
 *     "list_builder" = "Drupal\membership\Entity\Member\MemberListBuilder",
 *     "form" = {
 *       "default" = "Drupal\membership\Entity\Member\Form\MemberForm",
 *       "add" = "Drupal\membership\Entity\Member\Form\MemberForm",
 *       "edit" = "Drupal\membership\Entity\Member\Form\MemberForm",
 *       "delete" = "Drupal\membership\Entity\Member\Form\MemberRemoveForm",
 *     },
 *     "access" = "Drupal\membership\Entity\MembershipAccessControlHandler",
 *     "route_provider" = {
 *       "html" = "Drupal\membership\Entity\Member\MemberHtmlRouteProvider",
 *     },
 *   },
 *   base_table = "membership_member",
 *   admin_permission = "administer memberships",
 *   entity_keys = {
 *     "id" = "id",
 *     "uuid" = "uuid",
 *   },
 *   links = {
 *     "collection" = "/membership/{membership}/accounts",
 *     "canonical" = "/membership/{membership}/account/{membership_member}",
 *     "add-form" = "/membership/{membership}/account/add",
 *     "edit-form" = "/membership/{membership}/account/{membership_member}/edit",
 *     "delete-form" = "/membership/{membership}/account/{membership_member}/remove",
 *   },
 * )
 */
class Member extends ContentEntityBase implements MemberInterface {

  /**
   * {@inheritdoc}
   */
  public static function preCreate(EntityStorageInterface $storage_controller, array &$values) {
    parent::preCreate($storage_controller, $values);
  }

  /**
   * {@inheritdoc}
   */
  protected function urlRouteParameters($rel) {
    $uri_route_parameters = parent::urlRouteParameters($rel);

    // Add the membership id to the route parameters.
    $uri_route_parameters['membership'] = $this->memberId();
    return $uri_route_parameters;
  }

  /**
   * {@inheritdoc}
   */
  public function preSave(EntityStorageInterface $storage) {
    parent::preSave($storage);
  }

  /**
   *{@inheritdoc}
   */
  public function memberId() {
    return $this->get('member_id')->target_id;
  }

  /**
   *{@inheritdoc}
   */
  public function accountName($link = TRUE) {
    /* @var \Drupal\user\Entity\User $account */
    $account = $this->get('member')->referencedEntities()[0];
    $display_name = $account->getDisplayName();

    if ($link) {
      $url = Url::fromRoute('entity.user.canonical', ['user' => $account->id()]);
      $link = Link::fromTextAndUrl($display_name, $url)->toString();
      return $link;
    }

    return $display_name;
  }

  /**
   *{@inheritdoc}
   */
  public function roles() {
    return '<none>';
  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
    $fields = parent::baseFieldDefinitions($entity_type);

    $fields['member_id'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel('Membership')
      ->setDescription(t('The Membership ID'))
      ->setRequired(TRUE)
      ->setSetting('target_type', 'membership');

    $fields['member'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Member'))
      ->setDescription(t('Enter a username to select an existing user account.'))
      ->setRequired(TRUE)
      ->setSetting('target_type', 'user')
      ->setDisplayOptions('form', [
        'type' => 'entity_reference_default',
        'weight' => 1,
      ]);

    return $fields;
  }

}
