<?php

namespace Drupal\membership\Entity;

use Drupal\Core\Entity\RevisionableContentEntityBase;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\Entity\EntityChangedTrait;
use Drupal\Core\Entity\EntityTypeInterface;

/**
 * Defines the Membership entity.
 *
 * @ingroup membership
 *
 * @ContentEntityType(
 *   id = "membership",
 *   label = @Translation("Membership"),
 *   label_collection = @Translation("Membership List"),
 *   label_singular = @Translation("membership"),
 *   label_plural = @Translation("memberships"),
 *   label_count = @PluralTranslation(
 *     singular = "@count membership",
 *     plural = "@count memberships"
 *   ),
 *   bundle_label = @Translation("Membership type"),
 *   handlers = {
 *     "view_builder" = "Drupal\Core\Entity\EntityViewBuilder",
 *     "list_builder" = "Drupal\membership\Entity\MembershipListBuilder",
 *     "views_data" = "Drupal\membership\Entity\MembershipViewsData",
 *     "form" = {
 *       "default" = "Drupal\membership\Form\MembershipForm",
 *       "add" = "Drupal\membership\Form\MembershipForm",
 *       "edit" = "Drupal\membership\Form\MembershipForm",
 *       "delete" = "Drupal\membership\Form\MembershipDeleteForm",
 *     },
 *     "access" = "Drupal\membership\Entity\MembershipAccessControlHandler",
 *     "route_provider" = {
 *       "html" = "Drupal\membership\Entity\MembershipHtmlRouteProvider",
 *     },
 *   },
 *   base_table = "membership",
 *   revision_table = "membership_revision",
 *   data_table = "membership_field_data",
 *   translatable = FALSE,
 *   admin_permission = "administer memberships",
 *   entity_keys = {
 *     "id" = "member_id",
 *     "revision" = "vid",
 *     "bundle" = "type",
 *     "uuid" = "uuid",
 *   },
 *   revision_metadata_keys = {
 *     "revision_user" = "revision_user",
 *     "revision_created" = "revision_created",
 *     "revision_log_message" = "revision_log",
 *   },
 *   links = {
 *     "collection" = "/admin/memberships/list",
 *     "add-page" = "/admin/memberships/add",
 *     "add-form" = "/admin/memberships/add/{membership_type}",
 *     "canonical" = "/membership/{membership}",
 *     "edit-form" = "/membership/{membership}/edit",
 *     "delete-form" = "/membership/{membership}/delete",
  *   },
 *   bundle_entity_type = "membership_type",
 *   field_ui_base_route = "entity.membership_type.edit_form",
 *   show_revision_ui = TRUE,
 *   common_reference_target = TRUE
 * )
 */
class Membership extends RevisionableContentEntityBase  implements MembershipInterface {
  use EntityChangedTrait;

  /**
   * {@inheritdoc}
   */
  public static function preCreate(EntityStorageInterface $storage_controller, array &$values) {
    parent::preCreate($storage_controller, $values);
  }

  /**
   * {@inheritdoc}
   */
  protected function urlRouteParameters($rel) {
    $uri_route_parameters = parent::urlRouteParameters($rel);

    if ($rel === 'revision_revert' && $this instanceof RevisionableInterface) {
      $uri_route_parameters[$this->getEntityTypeId() . '_revision'] = $this->getRevisionId();
    }
    elseif ($rel === 'revision_delete' && $this instanceof RevisionableInterface) {
      $uri_route_parameters[$this->getEntityTypeId() . '_revision'] = $this->getRevisionId();
    }

    return $uri_route_parameters;
  }

  /**
   * {@inheritdoc}
   */
  public function preSave(EntityStorageInterface $storage) {
    parent::preSave($storage);
  }

  /**
   * {@inheritdoc}
   */
  public function getMemberID(): string {
    return $this->get('member_id')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setMemberID(string $member_id): MembershipInterface {
    $this->set('member_id', $member_id);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getCreatedTime(): int {
    return $this->get('created')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setCreatedTime(int $timestamp): MembershipInterface {
    $this->set('created', $timestamp);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
    $fields = parent::baseFieldDefinitions($entity_type);

    $fields['member_id'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Member ID'))
      ->setDescription(t('The unique Membership identifier.'))
      ->setSettings([
        // Hard limit is 191 due to MySQL <= 5.6 key size limit of 767 bytes.
        // 50 should be plenty.
        'max_length' => 50,
        'text_processing' => 0,
      ])
      ->setDefaultValue('')
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'string',
        'weight' => -4,
      ])
      ->setDisplayOptions('form', [
        'type' => 'string_textfield',
        'weight' => -4,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE)
      ->setRequired(TRUE);

    $fields['created'] = BaseFieldDefinition::create('created')
      ->setLabel(t('Created'))
      ->setRevisionable(TRUE)
      ->setDescription(t('The time that the entity was created.'));

    $fields['changed'] = BaseFieldDefinition::create('changed')
      ->setLabel(t('Changed'))
      ->setRevisionable(TRUE)
      ->setDescription(t('The time that the entity was last edited.'));

    $fields['membership_option'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Membership Option'))
      ->setDescription(t('Select the membership option. This option will be applied for all future renewals but will not affect past membership terms.'))
      ->setRequired(TRUE)
      ->setSetting('target_type', 'membership_option')
      ->setDisplayOptions('form', [
        'type' => 'entity_reference_default',
        'weight' => 2,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    return $fields;
  }

}
