<?php

namespace Drupal\membership\Entity\MembershipOption\Form;

use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Form\FormStateInterface;

/**
 * Form controller for Membership Term forms.
 *
 * @ingroup membership
 */
class MembershipOptionForm extends EntityForm {

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);
    $form['#tree'] = TRUE;
    $form['#attached']['library'][] = 'membership/options_form';

    /** @var \Drupal\membership\Entity\MembershipOption\MembershipOption $option */
    $option = $this->entity;
    $settings = $option->getSettings();

    if ($this->operation == 'add') {
      $form['#title'] = $this->t('Add membership option');
    }
    else {
      $form['#title'] = $this->t('Edit %label membership option', ['%label' => $option->label()]);
    }

    $form['label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Label'),
      '#size' => 30,
      '#maxlength' => 255,
      '#default_value' => $option->label(),
      '#description' => $this->t('This will be displayed as part of the list of available membership options on the <em>join</em> page.'),
      '#required' => TRUE,
    ];

    $form['id'] = [
      '#type' => 'machine_name',
      '#maxlength' => EntityTypeInterface::BUNDLE_MAX_LENGTH,
      '#default_value' => $option->id(),
      '#machine_name' => [
        'exists' => '\Drupal\membership\Entity\MembershipOption\MembershipOption::load',
      ],
      '#disabled' => !$option->isNew(),
      '#description' => $this->t('A unique machine-readable name. It must only contain lowercase letters, numbers, and underscores.'),
    ];

    // @TODO Timezone handling.

    $form['settings']['interval'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Interval'),
      '#description' => $this->t('Select the type and duration of membership for this option.'),
    ];
    $form['settings']['interval']['interval_type'] = [
      '#type' => 'radios',
      '#title' => $this->t('Type'),
      '#options' => [
        'rolling' => $this->t('Rolling Start'),
        'fixed' => $this->t('Fixed Start'),
      ],
      '#default_value' => isset($settings['interval']['interval_type']) ? $settings['interval']['interval_type'] : 'rolling',
    ];
    $form['settings']['interval']['number'] = [
      '#type' => 'number',
      '#title' => $this->t('Duration'),
      '#min' => 1,
      '#default_value' => isset($settings['interval']['number']) ? $settings['interval']['number'] : 1,
    ];
    $form['settings']['interval']['unit'] = [
      '#type' => 'select',
      '#options' => [
        'hour' => $this->t('Hour(s)'),
        'day' => $this->t('Day(s)'),
        'week' => $this->t('Week(s)'),
        'month' => $this->t('Month(s)'),
        'year' => $this->t('Year(s)'),
        'lifetime' => $this->t('Lifetime')
      ],
      '#default_value' => isset($settings['interval']['unit']) ? $settings['interval']['unit'] : 'month',
    ];

    $form['settings']['interval']['week_day'] = [
      '#title' => $this->t('Start Day'),
      '#type' => 'select',
      '#options' => [
        'mon' => $this->t('Monday'),
        'tues' => $this->t('Tuesday'),
        'wed' => $this->t('Wednesday'),
        'thu' => $this->t('Thursday'),
        'fri' => $this->t('Friday'),
        'sat' => $this->t('Saturday'),
        'sun' => $this->t('Sunday'),
      ],
      '#default_value' => isset($settings['interval']['week_day']) ? $settings['interval']['week_day'] : '',
      '#states' => [
        'visible' => [
          ':input[name="settings[interval][interval_type]"]' => ['value' => 'fixed'],
          ':input[name="settings[interval][unit]"]' => ['value' => 'week'],
        ],
      ],
    ];

    $form['settings']['interval']['start_month'] = [
      '#title' => $this->t('Start Month'),
      '#type' => 'select',
      '#options' => [
        1 => $this->t('January'),
        2 => $this->t('February'),
        3 => $this->t('March'),
        4 => $this->t('April'),
        5 => $this->t('May'),
        6 => $this->t('June'),
        7 => $this->t('July'),
        8 => $this->t('August'),
        9 => $this->t('September'),
        10 => $this->t('October'),
        11 => $this->t('November'),
        12 => $this->t('December'),
      ],
      '#default_value' => isset($settings['interval']['start_month']) ? $settings['interval']['start_month'] : '',
      '#states' => [
        'visible' => [
          ':input[name="settings[interval][interval_type]"]' => ['value' => 'fixed'],
          ':input[name="settings[interval][unit]"]' => ['value' => 'year'],
        ],
      ],
    ];

    $form['settings']['interval']['start_day'] = [
      '#title' => $this->t('Start Day'),
      '#type' => 'select',
      '#options' => range(1, 31),
      '#default_value' => isset($settings['interval']['start_day']) ? $settings['interval']['start_day'] : '',
      '#states' => [
        'visible' => [
          ':input[name="settings[interval][interval_type]"]' => ['value' => 'fixed'],
          ':input[name="settings[interval][unit]"]' => [
            ['value' => 'month'],
            ['value' => 'year'],
          ],
        ],
      ],
    ];

    $form['settings']['price'] = [
      '#type' => 'number',
      '#title' => $this->t('Price'),
      '#field_prefix' => '$',
      '#min' => 0,
      '#default_value' => isset($settings['price']) ? $settings['price'] : 0,
    ];

    $form['settings']['prorated'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Prorated'),
      '#default_value' => isset($settings['prorated']) ? $settings['prorated'] : 0,
      '#states' => [
        'visible' => [
          ':input[name="settings[interval][interval_type]"]' => ['value' => 'fixed'],
        ],
      ],
    ];

    $form['settings']['grace_period'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Grace Period'),
      '#description' => $this->t('Select a Grace Period for this option. Memberships will not become inactive until after the grace period following term expiration.'),
    ];
    $form['settings']['grace_period']['number'] = [
      '#type' => 'number',
      '#title' => $this->t('Duration'),
      '#min' => 0,
      '#default_value' => isset($settings['grace_period']['number']) ? $settings['grace_period']['number'] : '',
    ];
    $form['settings']['grace_period']['unit'] = [
      '#type' => 'select',
      '#options' => [
        'hour' => $this->t('Hour(s)'),
        'day' => $this->t('Day(s)'),
        'week' => $this->t('Week(s)'),
        'month' => $this->t('Month(s)'),
        'year' => $this->t('Year(s)')
      ],
      '#default_value' => isset($settings['grace_period']['unit']) ? $settings['grace_period']['unit'] : 'day',
    ];

    $form['status'] = [
      '#type' => 'select',
      '#title' => $this->t('Status'),
      '#options' => [
        0 => $this->t('Disabled'),
        1 => $this->t('Enabled'),
      ],
      '#default_value' => $option->status() ?? 0,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    $values = $form_state->getValues();

    /** @var \Drupal\membership\Entity\MembershipOption\MembershipOption $option */
    $option = $this->entity;
    $option->setSettings($values['settings']);

    $status = $option->save();

    $params = ['%label' => $option->label()];
    $messages = [
      SAVED_NEW => $this->t('Created the %label Membership option.', $params),
      'default' => $this->t('Saved the %label Membership option.', $params),
    ];
    if (isset($messages[$status])) {
      $this->messenger()->addMessage($messages[$status]);
    }
    else {
      $this->messenger()->addMessage($messages['default']);
    }

    $form_state->setRedirectUrl($option->toUrl('collection'));
  }
}
