<?php
declare(strict_types=1);

namespace Drupal\membership\Entity\MembershipTerm;

use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Datetime\Entity\DateFormat;
use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\Entity\EntityTypeInterface;

/**
 * Defines the Membership Term entity.
 *
 * @ingroup membership
 *
 * @ContentEntityType(
 *   id = "membership_term",
 *   label = @Translation("Membership Term"),
 *   label_collection = @Translation("Membership History"),
 *   label_singular = @Translation("membership term"),
 *   label_plural = @Translation("membership terms"),
 *   label_count = @PluralTranslation(
 *     singular = "@count membership term",
 *     plural = "@count membership terms"
 *   ),
 *   handlers = {
 *     "view_builder" = "Drupal\Core\Entity\EntityViewBuilder",
 *     "list_builder" = "Drupal\membership\Entity\MembershipTerm\MembershipTermListBuilder",
 *     "form" = {
 *       "default" = "Drupal\membership\Entity\MembershipTerm\Form\MembershipTermForm",
 *       "add" = "Drupal\membership\Entity\MembershipTerm\Form\MembershipTermForm",
 *       "edit" = "Drupal\membership\Entity\MembershipTerm\Form\MembershipTermForm",
 *       "delete" = "Drupal\membership\Entity\MembershipTerm\Form\MembershipTermDeleteForm",
 *     },
 *     "access" = "Drupal\membership\Entity\MembershipAccessControlHandler",
 *     "route_provider" = {
 *       "html" = "Drupal\membership\Entity\MembershipTerm\MembershipTermHtmlRouteProvider",
 *     },
 *   },
 *   base_table = "membership_term",
 *   admin_permission = "administer memberships",
 *   entity_keys = {
 *     "id" = "id",
 *     "uuid" = "uuid",
 *   },
 *   links = {
 *     "collection" = "/membership/{membership}/history",
 *     "canonical" = "/membership/{membership}/term/{membership_term}",
 *     "add-form" = "/membership/{membership}/term/add",
 *     "edit-form" = "/membership/{membership}/term/{membership_term}/edit",
 *     "delete-form" = "/membership/{membership}/term/{membership_term}/delete",
 *   },
 * )
 */
class MembershipTerm extends ContentEntityBase implements MembershipTermInterface {

  /**
   * {@inheritdoc}
   */
  public static function preCreate(EntityStorageInterface $storage_controller, array &$values) {
    parent::preCreate($storage_controller, $values);
  }

  /**
   * {@inheritdoc}
   */
  protected function urlRouteParameters($rel) {
    $uri_route_parameters = parent::urlRouteParameters($rel);

    // Add the membership id to the route parameters.
    $uri_route_parameters['membership'] = $this->memberId();
    return $uri_route_parameters;
  }

  /**
   * {@inheritdoc}
   */
  public function preSave(EntityStorageInterface $storage) {
    parent::preSave($storage);
  }

  /**
   * {@inheritdoc}
   */
  public function memberId() {
    return $this->get('member_id')->target_id;
  }

  /**
   * {@inheritdoc}
   */
  public function term() {
    return $this->get('term');
  }

  /**
   * @{inheritdoc}
   */
  public function isLifetime() {
    return $this->get('term')->term_length === 'lifetime';
  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
    $fields = parent::baseFieldDefinitions($entity_type);

    $fields['member_id'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel('Membership')
      ->setDescription(t('The Membership ID'))
      ->setRequired(TRUE)
      ->setSetting('target_type', 'membership');

    $fields['membership_option'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Term Option'))
      ->setDescription(t('Select the membership term option.'))
      ->setRequired(TRUE)
      ->setSetting('target_type', 'membership_option')
      ->setDisplayOptions('form', [
        'type' => 'entity_reference_default',
        'weight' => 2,
      ]);

    $fields['status'] = BaseFieldDefinition::create('membership_workflow')
      ->setLabel(t('Status'))
      ->setDescription(t('The current membership term workflow state.'))
      ->setRequired(TRUE)
      ->setDisplayOptions('form', [
        'type' => 'membership_workflow_default',
        'weight' => 0,
      ]);

    $fields['term'] = BaseFieldDefinition::create('membership_term_length')
      ->setLabel(t('Term Dates'))
      ->setDescription(t('The membership term length including start and end dates.'))
      ->setRequired(TRUE)
      ->setDisplayOptions('form', [
        'type' => 'membership_term_length_default',
        'weight' => 1,
      ]);

    return $fields;
  }

}
