<?php
declare(strict_types=1);

namespace Drupal\membership\Plugin\Field\FieldType;

use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\TypedData\DataDefinition;
use Drupal\datetime\Plugin\Field\FieldType\DateTimeItemInterface;
use Drupal\datetime_range\Plugin\Field\FieldType\DateRangeItem;

/**
 * Plugin implementation of the 'membership_term_length' field type.
 *
 * @FieldType(
 *   id = "membership_term_length",
 *   label = @Translation("Term length"),
 *   description = @Translation("Create and store term length with comupted date ranges."),
 *   default_widget = "membership_term_length_default",
 *   default_formatter = "membership_term_length_default",
 *   no_ui = "true"
 * )
 */
class TermLengthItem extends DateRangeItem {

  /**
   * {@inheritdoc}
   */
  public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition) {
    $properties = parent::propertyDefinitions($field_definition);
    $properties['term_length'] = DataDefinition::create('string')
      ->setLabel(t('Term length value'))
      ->setRequired(TRUE);
    $properties['end_value']->setRequired(FALSE);
    return $properties;
  }

  /**
   * {@inheritdoc}
   */
  public static function schema(FieldStorageDefinitionInterface $field_definition) {
    $schema = parent::schema($field_definition);
    $schema['columns']['term_length'] = [
      'description' => 'The term length value.',
      'type' => 'varchar',
      'length' => 255,
    ];
    return $schema;
  }

  /**
   * {@inheritdoc}
   */
  public static function generateSampleValue(FieldDefinitionInterface $field_definition) {
    $type = $field_definition
      ->getSetting('datetime_type');

    // Just pick a date in the past year. Term length is 1 year.
    $values['term_length'] = '1 year';
    $start = \Drupal::time()->getRequestTime() - mt_rand(0, 86400 * 365) - 86400;
    $end = strtotime($values['term_length'], $start);
    if ($type == static::DATETIME_TYPE_DATETIME) {

      $values['value'] = gmdate(DateTimeItemInterface::DATETIME_STORAGE_FORMAT, $start);
      $values['end_value'] = gmdate(DateTimeItemInterface::DATETIME_STORAGE_FORMAT, $end);
    }
    else {
      $values['value'] = gmdate(DateTimeItemInterface::DATE_STORAGE_FORMAT, $start);
      $values['end_value'] = gmdate(DateTimeItemInterface::DATE_STORAGE_FORMAT, $end);
    }
    return $values;
  }

  /**
   * {@inheritdoc}
   */
  public function isEmpty() {
    $term_length = $this->get('term_length')->getValue();
    $start_value = $this->get('value')->getValue();
    $end_value = $this->get('end_value')->getValue();
    return ($term_length === NULL || $term_length === '') &&
      ($start_value === NULL || $start_value === '') &&
      ($end_value === NULL || $end_value === '');
  }

  /**
   * {@inheritdoc}
   */
  public function onChange($property_name, $notify = TRUE) {
    //@TODO Ensure that the computed ed date is recalculated.
    parent::onChange($property_name, $notify);
  }

}