<?php
declare(strict_types=1);

namespace Drupal\membership\Plugin\Field\FieldWidget;

use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\datetime_range\Plugin\Field\FieldWidget\DateRangeDefaultWidget;

/**
 * Plugin implementation of the 'daterange_default' widget.
 *
 * @FieldWidget(
 *   id = "membership_term_length_default",
 *   label = @Translation("Date and time range"),
 *   field_types = {
 *     "membership_term_length"
 *   }
 * )
 */
class TermLengthDefaultWidget extends DateRangeDefaultWidget {

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $element = parent::formElement($items, $delta, $element, $form, $form_state);

    $element['term_length'] = [
      '#type' => 'select',
      '#title' => $this->t('Term length'),
      '#required' => TRUE,
      '#weight' => 0,
      '#options' => [
        '1 year' => $this->t('1 Year'),
        '2 years' => $this->t('2 Years'),
        '3 years' => $this->t('3 Years'),
        '1 month' => $this->t('1 Month'),
        '3 months' => $this->t('3 Months'),
        '6 months' => $this->t('6 Months'),
        'lifetime' => $this->t('Lifetime'),
        'custom' => $this->t('Custom'),
      ],
    ];

    $element['value']['#weight'] = 1;

    $element['end_value']['#required'] = FALSE;
    $element['end_value']['#weight'] = 2;

    // Calculate end date on submission.
    $element['#element_validate'][] = [$this, 'calculateEndDate'];

    // @TODO Known bug with #states on datetime elements.
    // see https://www.drupal.org/project/drupal/issues/2419131
    $field_name = $this->fieldDefinition->getName();
    $element['end_value']['#states'] = [
      'visible' => [
        ':input[name="' . $field_name . '[' . $delta . '][term_length]"]' => ['value' => 'custom'],
      ],
      'required' => [
        ':input[name="' . $field_name . '[' . $delta . '][term_length]"]' => ['value' => 'custom'],
      ]
    ];

    return $element;
  }

  /**
   * #element_validate callback to calculate the term end date.
   *
   * End date is based on start date and term length.
   *
   * @param array $element
   *   An associative array containing the properties and children of the
   *   generic form element.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   * @param array $complete_form
   *   The complete form structure.
   */
  public function calculateEndDate(array &$element, FormStateInterface $form_state, array &$complete_form) {
    $term_length = $element['term_length']['#value'];

    /** @var \Drupal\Core\Datetime\DrupalDateTime $start_date */
    $start_date = $element['value']['#value']['object'];

    if ($term_length === 'lifetime') {
      return;
    }

    if ($start_date instanceof DrupalDateTime && $term_length !== 'custom') {
      /** @var \Drupal\Core\Datetime\DrupalDateTime $end_date */
      $end_date = clone $start_date;
      $end_date->modify($term_length);
      $form_state->setValueForElement($element['end_value'], $end_date);
    }
  }

}