<?php
declare(strict_types=1);

namespace Drupal\membership\Plugin\WorkflowType;

use Drupal\content_moderation\ModerationInformationInterface;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\workflows\Plugin\WorkflowTypeBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Workflows for Memberships.
 *
 * @WorkflowType(
 *   id = "membership_status",
 *   label = @Translation("Membership Status"),
 *   required_states = {
 *     "active",
 *     "expired",
 *   },
 * )
 */
class MembershipStatus extends WorkflowTypeBase implements ContainerFactoryPluginInterface {
  use StringTranslationTrait;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The entity type bundle info service.
   *
   * @var \Drupal\Core\Entity\EntityTypeBundleInfoInterface
   */
  protected $entityTypeBundleInfo;

  /**
   * The moderation information service.
   *
   * @var \Drupal\content_moderation\ModerationInformationInterface
   */
  protected $moderationInfo;

  /**
   * Constructs a ContentModeration object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\content_moderation\ModerationInformationInterface $moderation_info
   *   Moderation information service.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entity_type_manager, EntityTypeBundleInfoInterface $entity_type_bundle_info, ModerationInformationInterface $moderation_info) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->entityTypeManager = $entity_type_manager;
    $this->entityTypeBundleInfo = $entity_type_bundle_info;
    $this->moderationInfo = $moderation_info;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('entity_type.bundle.info'),
      $container->get('content_moderation.moderation_information')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'states' => [
        'pending' => [
          'label' => $this->t('Pending'),
          'weight' => 0,
        ],
        'active' => [
          'label' => $this->t('Active'),
          'weight' => 1,
        ],
        'expired' => [
          'label' => $this->t('Expired'),
          'weight' => 2,
        ],
        'cancelled' => [
          'label' => $this->t('Cancelled'),
          'weight' => 3,
        ],
      ],
      'transitions' => [
        'activate' => [
          'label' => $this->t('Activate'),
          'weight' => 0,
          'to' => 'active',
          'from' => [
            'pending',
            'cancelled',
          ],
        ],
        'cancel' => [
          'label' => $this->t('Cancel'),
          'weight' => 1,
          'to' => 'cancelled',
          'from' => [
            'pending',
            'active',
            'expired'
          ],
        ],
        'expire' => [
          'label' => $this->t('Expire'),
          'weight' => 2,
          'to' => 'expired',
          'from' => ['active'],
        ],
      ],
    ];
  }
}